@@ -3,6 +3,7 @@ import { config } from '@grafana/runtime';
 
 import { LOG_LINE_BODY_FIELD_NAME } from '../LogDetailsBody';
 import { createLogLine, createLogRow } from '../mocks/logRow';
+import { OTEL_LOG_LINE_ATTRIBUTES_FIELD_NAME, OTEL_PROBE_FIELD } from '../otel/formats';
 
 import { LogListFontSize } from './LogList';
 import { LogListModel, preProcessLogs } from './processing';
@@ -237,6 +238,67 @@ describe('preProcessLogs', () => {
       expect(logListModel.body).toBeDefined(); // Triggers parsing
       expect(logListModel.isJSON).toBe(false);
     });
+
+    describe('OTel logs', () => {
+      const originalState = config.featureToggles.otelLogsFormatting;
+
+      test('Does not create the OTel attribute field when not enabled', () => {
+        config.featureToggles.otelLogsFormatting = false;
+
+        const logListModel = createLogLine(
+          { entry: 'the log' },
+          {
+            escape: false,
+            order: LogsSortOrder.Descending,
+            timeZone: 'browser',
+            wrapLogMessage: true, // wrapped
+            prettifyJSON: true,
+          }
+        );
+        expect(logListModel.labels[OTEL_LOG_LINE_ATTRIBUTES_FIELD_NAME]).toBeUndefined();
+        expect(logListModel.highlightedLogAttributesTokens).toHaveLength(0);
+
+        config.featureToggles.otelLogsFormatting = originalState;
+      });
+
+      test('Does not create the OTel attribute field when is not an OTel log', () => {
+        config.featureToggles.otelLogsFormatting = false;
+
+        const logListModel = createLogLine(
+          { entry: 'the log', labels: {} },
+          {
+            escape: false,
+            order: LogsSortOrder.Descending,
+            timeZone: 'browser',
+            wrapLogMessage: true, // wrapped
+            prettifyJSON: true,
+          }
+        );
+        expect(logListModel.labels[OTEL_LOG_LINE_ATTRIBUTES_FIELD_NAME]).toBeUndefined();
+        expect(logListModel.highlightedLogAttributesTokens).toHaveLength(0);
+
+        config.featureToggles.otelLogsFormatting = originalState;
+      });
+
+      test('Generates and highlights an OTel log line attributes field', () => {
+        config.featureToggles.otelLogsFormatting = true;
+
+        const logListModel = createLogLine(
+          { entry: 'the log', labels: { [OTEL_PROBE_FIELD]: '1', field: 'value' } },
+          {
+            escape: false,
+            order: LogsSortOrder.Descending,
+            timeZone: 'browser',
+            wrapLogMessage: true, // wrapped
+            prettifyJSON: true,
+          }
+        );
+        expect(logListModel.labels[OTEL_LOG_LINE_ATTRIBUTES_FIELD_NAME]).toEqual('field=value');
+        expect(logListModel.highlightedLogAttributesTokens).toHaveLength(2);
+
+        config.featureToggles.otelLogsFormatting = originalState;
+      });
+    });
   });
 
   test('Orders logs', () => {
@@ -283,21 +345,47 @@ describe('preProcessLogs', () => {
   });
 
   test('Highlights tokens in log lines', () => {
-    expect(processedLogs[0].highlightedBody).toContain('log-token-label');
-    expect(processedLogs[0].highlightedBody).toContain('log-token-key');
-    expect(processedLogs[0].highlightedBody).toContain('log-token-string');
-    expect(processedLogs[0].highlightedBody).toContain('log-token-uuid');
-    expect(processedLogs[0].highlightedBody).not.toContain('log-token-method');
-    expect(processedLogs[0].highlightedBody).not.toContain('log-token-json-key');
+    expect(processedLogs[0].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-label' })])
+    );
+    expect(processedLogs[0].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-key' })])
+    );
+    expect(processedLogs[0].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-string' })])
+    );
+    expect(processedLogs[0].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-uuid' })])
+    );
+    expect(processedLogs[0].highlightedBodyTokens).not.toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-method' })])
+    );
+    expect(processedLogs[0].highlightedBodyTokens).not.toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-json-key' })])
+    );
 
-    expect(processedLogs[1].highlightedBody).toContain('log-token-method');
-    expect(processedLogs[1].highlightedBody).toContain('log-token-key');
-    expect(processedLogs[1].highlightedBody).toContain('log-token-string');
-    expect(processedLogs[1].highlightedBody).not.toContain('log-token-json-key');
+    expect(processedLogs[1].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-method' })])
+    );
+    expect(processedLogs[1].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-key' })])
+    );
+    expect(processedLogs[1].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-string' })])
+    );
+    expect(processedLogs[1].highlightedBodyTokens).not.toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-json-key' })])
+    );
 
-    expect(processedLogs[2].highlightedBody).toContain('log-token-json-key');
-    expect(processedLogs[2].highlightedBody).toContain('log-token-string');
-    expect(processedLogs[2].highlightedBody).not.toContain('log-token-method');
+    expect(processedLogs[2].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-json-key' })])
+    );
+    expect(processedLogs[2].highlightedBodyTokens).toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-string' })])
+    );
+    expect(processedLogs[2].highlightedBodyTokens).not.toEqual(
+      expect.arrayContaining([expect.objectContaining({ type: 'log-token-method' })])
+    );
   });
 
   test('Returns displayed field values', () => {
@@ -414,43 +502,3 @@ describe('preProcessLogs', () => {
     });
   });
 });
-
-describe('OTel logs', () => {
-  let originalOtelLogsFormatting = config.featureToggles.otelLogsFormatting;
-  afterAll(() => {
-    config.featureToggles.otelLogsFormatting = originalOtelLogsFormatting;
-  });
-
-  test('Requires a feature flag', () => {
-    const log = createLogLine({
-      labels: {
-        severity_number: '1',
-        telemetry_sdk_language: 'php',
-        scope_name: 'scope',
-        aws_ignore: 'ignored',
-        key: 'value',
-        otel: 'otel',
-      },
-      entry: `place="luna" 1ms 3 KB`,
-    });
-    expect(log.otelLanguage).toBeDefined();
-    expect(log.body).toEqual(`place="luna" 1ms 3 KB`);
-  });
-
-  test('Augments OTel log lines', () => {
-    config.featureToggles.otelLogsFormatting = true;
-    const log = createLogLine({
-      labels: {
-        severity_number: '1',
-        telemetry_sdk_language: 'php',
-        scope_name: 'scope',
-        aws_ignore: 'ignored',
-        key: 'value',
-        otel: 'otel',
-      },
-      entry: `place="luna" 1ms 3 KB`,
-    });
-    expect(log.otelLanguage).toBeDefined();
-    expect(log.body).toEqual(`place="luna" 1ms 3 KB key=value otel=otel`);
-  });
-});
